// Copyright (c) 2015, Mikhail Stolyarov. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.


library do2init.base;

import 'dart:async';
import 'do2loader.dart';

class Do2Init {
  
  static List<Do2Loader> _modules = new List();
  
  static Future loading(List<Do2Loader> modules, {Function progress(int value, int max, dynamic result): null}){
    List<Future> wait = _loading(modules);
    return _process(wait, progress);
  }
  
  static Future unloading(List<Do2Loader> modules,{Function progress(int value, int max, dynamic result): null} ){
    List<Future> wait = _unloading(modules);
    return _process(wait, progress);
  }
  
  static Future unloadingAll({Function progress(int value, int max, dynamic result): null}){
    List<Do2Loader> modules = new List();
    _modules.forEach((Do2Loader m)=>modules.add(m));
    List<Future> wait = _unloading(modules);
    return _process(wait, progress);
  }
  
  static List<Future> _loading(List<Do2Loader> modules){
    List<Future> wait = new List();
    modules.forEach((Do2Loader module){
      if( ! _modules.contains(module)){
        _modules.add(module);
        Completer moduleCompleter = new Completer();
        if (module.isLoadWait) wait.add(moduleCompleter.future);
        module.loading(moduleCompleter);
      }
    });
    return wait;
  }
  
  static List<Future> _unloading(List<Do2Loader> modules){
    List<Future> wait = new List();
    modules.forEach((Do2Loader module){
      if(_modules.contains(module)){
        _modules.remove(module);
        Completer moduleCompleter = new Completer();
        if (module.isUnloadWait) wait.add(moduleCompleter.future);
        module.unloading(moduleCompleter);
      }
    });
    return wait;
  }
  
  static Future<List> _process(List<Future> wait, Function progress){
    Completer<List<Do2Loader>> completer = new Completer();
    
    if(progress != null){
      int progressLoadValue = 0;
      wait.forEach((Future future)=> future.then((result)=>progress(++progressLoadValue, wait.length, result)));
    }
    
    Future.wait(wait).then((List value) {
      completer.complete(value);
    });
    
    return completer.future;
  }
}
