// Copyright (c) 2015, Mikhail Stolyarov. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

library do2init.test;

import 'package:test/test.dart';
import 'package:do2init/do2init.dart';
import 'dart:async';

part 'classes/module1.dart';
part 'classes/module2.dart';
part 'classes/observable.dart';
part 'classes/observer.dart';

main() {
  
  group('Do2Init', (){
    
    Module1 mod1 = new Module1();
    Module2 mod2 = new Module2();
    
    test('Load', (){
      Function completeLoad = expectAsync((List value) {
        expect(value, equals(["loading","loading"]));
      }, count:1, max:1);
      Function progressLoad = expectAsync((int value, int max, dynamic result) {
        expect(value, inInclusiveRange(1,2));
        expect(max, equals(2));
        expect(result, equals("loading"));
      }, count:2, max:2);
      Do2Init.loading([mod1, mod2], progress:progressLoad).then(completeLoad);
      
      Function completeLoad2 = expectAsync((List value) {
        expect(value, equals([]));
      }, count:1, max:1);
      Function progressLoad2 = expectAsync((int value, int max, dynamic result) {
      }, count:0, max:0);
      Do2Init.loading([mod1, mod2], progress:progressLoad2).then(completeLoad2);
    });
    
    test('Unload', (){
      Function completeUnload = expectAsync((List value) {
        expect(value, equals(["unloading","unloading"]));
      }, count:1, max:1);
      Function progressUnload = expectAsync((int value, int max, dynamic result) {
        expect(value, inInclusiveRange(1,2));
        expect(max, equals(2));
        expect(result, equals("unloading"));
      }, count:2, max:2);
      Do2Init.unloading([mod1, mod2], progress:progressUnload).then(completeUnload);
      
      Function completeUnload2 = expectAsync((List value) {
        expect(value, equals([]));
      }, count:1, max:1);
      Function progressUnload2 = expectAsync((int value, int max, dynamic result) {
      }, count:0, max:0);
      Do2Init.unloading([mod1, mod2], progress:progressUnload2).then(completeUnload2);
    });
    
    test('Unload all', (){
      Do2Init.loading([mod1, mod2]);
      Function completeUnload = expectAsync((List value) {
        expect(value, equals(["unloading","unloading"]));
      }, count:1, max:1);
      Function progressUnload = expectAsync((int value, int max, dynamic result) {
        expect(value, inInclusiveRange(1,2));
        expect(max, equals(2));
        expect(result, equals("unloading"));
      }, count:2, max:2);
      Do2Init.unloadingAll(progress:progressUnload).then(completeUnload);
    });
  });
  
  test('Do2Observer', (){
    Observer observer = new Observer();
    expect(observer, isNotNull);
    Observable observable = new Observable();
    expect(observable, isNotNull);
    expect(observer.val, isNull);
    observable.notify();
    observable.attach(observer);
    observable.notify();
    expect(observer.val, same(observable));
    observer.val = null;
    observable.detach(observer);
    observable.notify();
    expect(observer.val, isNull);
  });
    
}
